"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommitsForRepository = void 0;
var fs = require("fs");
var isoGit = require("isomorphic-git");
var git_utils_1 = require("./git.utils");
function getBranchNames(args) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, isoGit.listBranches({ fs: fs, dir: args.gitPath })];
        });
    });
}
function getCommitsForRepository(_a) {
    var gitPath = _a.gitPath, countMerges = _a.countMerges, since = _a.since, until = _a.until;
    return __awaiter(this, void 0, void 0, function () {
        var branchNames, allCommits, cache, branchCommits, uniqueCommits, isNotMergeCommit;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, getBranchNames({
                        gitPath: gitPath,
                        countMerges: countMerges,
                        since: since,
                        until: until,
                    })];
                case 1:
                    branchNames = _b.sent();
                    allCommits = [];
                    cache = {};
                    return [4 /*yield*/, Promise.all(branchNames.map(function (ref) {
                            return getBranchCommits({ ref: ref, since: since, until: until, dir: gitPath, cache: cache });
                        }))];
                case 2:
                    branchCommits = _b.sent();
                    branchCommits.forEach(function (commitList) {
                        return commitList.forEach(function (commit) { return allCommits.push(commit); });
                    });
                    uniqueCommits = allCommits.filter(git_utils_1.generateUniquesBasedOnKeyFilter('sha'));
                    if (countMerges) {
                        return [2 /*return*/, uniqueCommits];
                    }
                    isNotMergeCommit = function (commit) { return !commit.message.startsWith('Merge '); };
                    return [2 /*return*/, uniqueCommits.filter(isNotMergeCommit)];
            }
        });
    });
}
exports.getCommitsForRepository = getCommitsForRepository;
function getBranchCommits(_a) {
    var ref = _a.ref, dir = _a.dir, since = _a.since, until = _a.until, cache = _a.cache;
    return __awaiter(this, void 0, void 0, function () {
        var commits, maxAge_1;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, isoGit.log({
                        ref: ref,
                        fs: fs,
                        dir: dir,
                        cache: cache,
                        since: since === 'always' ? undefined : since,
                    })];
                case 1:
                    commits = _b.sent();
                    if (until !== 'always') {
                        maxAge_1 = Math.ceil(until.getTime() / 1000);
                        commits = commits.filter(function (c) { return c.commit.committer.timestamp < maxAge_1; });
                    }
                    return [2 /*return*/, commits.map(function (c) { return ({
                            sha: c.oid,
                            author: { email: c.commit.author.email, name: c.commit.author.name },
                            date: new Date(c.commit.committer.timestamp * 1000),
                            message: c.commit.message,
                            repo: dir,
                        }); })];
            }
        });
    });
}
